//
//  ViewController.swift
//  TableView
//
//  Created by Valery Bashkatov on 03.01.16.
//  Copyright © 2016 Valery Bashkatov. All rights reserved.
//

import UIKit

class ViewController: UIViewController,
                      NSXMLParserDelegate,
                      UITableViewDelegate,
                      UITableViewDataSource {
    /*
    map.xml format:
    
    &lt;sitemap&gt;
      &lt;article&gt;
        &lt;title&gt;Задание атрибутов в Interface Builder&lt;/title&gt;
        &lt;url&gt;http://valery.bashkatov.org/paper/setting-attributes-in-interface-builder&lt;/url&gt;
        &lt;date&gt;2015-12-28&lt;/date&gt;
      &lt;/article&gt;
    
      &lt;article&gt;
        &lt;title&gt;Изображение в качестве цвета&lt;/title&gt;
        &lt;url&gt;http://valery.bashkatov.org/paper/image-as-color&lt;/url&gt;
        &lt;date&gt;2015-12-23&lt;/date&gt;
      &lt;/article&gt;
    &lt;/sitemap&gt;
    */
    struct Article {
        var title: String
        var url: String
        var date: String
    }
    
    class UITableViewSubtitleCell: UITableViewCell {
        override init(style: UITableViewCellStyle, reuseIdentifier: String?) {
            super.init(style: .Subtitle, reuseIdentifier: reuseIdentifier)
        }

        required init?(coder aDecoder: NSCoder) {
            super.init(coder: aDecoder)
        }
    }
    
    var articles = [Article]()
    var xmlParser: NSXMLParser!
    var xmlCurrentElement = ""
    var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Create table view with grouped style
        tableView = UITableView(frame: CGRect(x: 0, y: 0, width: 0, height: 0), style: .Grouped)

        // Register UITableViewSubtitleCell class
        tableView.registerClass(UITableViewSubtitleCell.self, forCellReuseIdentifier: "cell")
        
        // Set delegate for UITableViewDelegate methods
        tableView.delegate = self
        
        // And data source for UITableViewDataSource methods
        tableView.dataSource = self
        
        view.addSubview(tableView)
        
        // Add constraints
        tableView.translatesAutoresizingMaskIntoConstraints = false
        tableView.centerXAnchor.constraintEqualToAnchor(view.centerXAnchor).active = true
        tableView.centerYAnchor.constraintEqualToAnchor(view.centerYAnchor).active = true
        tableView.widthAnchor.constraintEqualToAnchor(view.widthAnchor).active = true
        tableView.heightAnchor.constraintEqualToAnchor(view.heightAnchor).active = true
        
        // Create XML parser object
        let url = NSURL(string: "http://valery.bashkatov.org/files/uitableview-basics/map.xml")!
        xmlParser = NSXMLParser(contentsOfURL: url)!
        
        // For NSXMLParserDelegate methods
        xmlParser.delegate = self
        
        xmlParser.parse()
    }
    
    // Hide status bar
    override func prefersStatusBarHidden() -> Bool {
        return true
    }
    
    // MARK: - NSXMLParserDelegate methods
    // Called when detects a new tag
    func parser(parser: NSXMLParser,
                didStartElement elementName: String,
                namespaceURI: String?,
                qualifiedName qName: String?,
                attributes attributeDict: [String : String]) {
                
        xmlCurrentElement = elementName
        
        if elementName == "article" {
            articles.append(Article(title: "", url: "", date: ""))
        }
    }
    
    // Called when the text found inside tag
    func parser(parser: NSXMLParser, foundCharacters string: String) {

        // Since the text can come in parts, collect it
        switch xmlCurrentElement {
        case "title": articles[articles.count - 1].title += string
        case "url": articles[articles.count - 1].url += string
        case "date": articles[articles.count - 1].date += string
        default: return
        }
    }
    
    // Called when xml document was successfully parsed
    func parserDidEndDocument(parser: NSXMLParser) {
        tableView.reloadData()
    }
    
    // MARK: - UITableViewDataSource methods
    // Create and fill cells
    func tableView(tableView: UITableView,
                   cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
                
        let cell = tableView.dequeueReusableCellWithIdentifier("cell", forIndexPath: indexPath)
        let row = indexPath.row
                    
        // Title as link
        cell.textLabel!.attributedText = NSAttributedString(string: articles[row].title,
                                   attributes: [NSLinkAttributeName: articles[row].url])
                
        cell.detailTextLabel!.text = "Дата публикации: \(articles[row].date)"

        return cell
    }
    
    // Set cells count
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return articles.count
    }
    
    // Set section title
    func tableView(tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return "Записи"
    }
    
    // Logic on row deleting
    func tableView(tableView: UITableView,
                   commitEditingStyle editingStyle: UITableViewCellEditingStyle,
                   forRowAtIndexPath indexPath: NSIndexPath) {
                
        if editingStyle == .Delete {
            articles.removeAtIndex(indexPath.row)
            tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Automatic)
        }
    }
}
